@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2010 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -55,6 +55,9 @@ static PrefParam param[] = {
 	/* Send */
 	{"save_message", "TRUE", &prefs_common.savemsg, P_BOOL},
 	{"filter_sent_message", "FALSE", &prefs_common.filter_sent, P_BOOL},
+	{"recipients_autoreg", "TRUE", &prefs_common.recipients_autoreg,
+	 P_BOOL},
+	{"show_send_dialog", "TRUE", &prefs_common.show_send_dialog, P_BOOL},
 
 	{"encoding_method", "0", &prefs_common.encoding_method, P_ENUM},
 	{"mime_filename_encoding_method", "0",
@@ -90,6 +93,8 @@ static PrefParam param[] = {
 	 P_BOOL},
 	{"inherit_recipient_on_self_reply", "FALSE",
 	 &prefs_common.inherit_recipient_on_self_reply, P_BOOL},
+	{"reply_address_only", "FALSE",
+	 &prefs_common.reply_address_only, P_BOOL},
 
 	{"show_ruler", "TRUE", &prefs_common.show_ruler, P_BOOL},
 
@@ -117,6 +122,12 @@ static PrefParam param[] = {
 	 &prefs_common.display_folder_unread, P_BOOL},
 	{"display_folder_num_columns", "FALSE",
 	 &prefs_common.display_folder_num_columns, P_BOOL},
+	{"folder_col_show_new", "FALSE",
+	 &prefs_common.folder_col_visible[1], P_BOOL},
+	{"folder_col_show_unread", "FALSE",
+	 &prefs_common.folder_col_visible[2], P_BOOL},
+	{"folder_col_show_total", "FALSE",
+	 &prefs_common.folder_col_visible[3], P_BOOL},
 
 	{"newsgroup_abbrev_len", "16", &prefs_common.ng_abbrev_len, P_INT},
 
@@ -388,10 +399,12 @@ static PrefParam param[] = {
 	{"separate_folder", "FALSE", &prefs_common.sep_folder, P_BOOL},
 	{"separate_message", "FALSE", &prefs_common.sep_msg, P_BOOL},
 
-	{"always_show_message_when_selected", "FALSE",
+	{"always_show_message_when_selected", "TRUE",
 	 &prefs_common.always_show_msg, P_BOOL},
 	{"open_unread_on_enter", "FALSE", &prefs_common.open_unread_on_enter,
 	 P_BOOL},
+	{"always_mark_read_on_show_msg", "FALSE",
+	 &prefs_common.always_mark_read_on_show_msg, P_BOOL},
 	{"remember_last_selected", "FALSE",
 	 &prefs_common.remember_last_selected, P_BOOL},
 	{"mark_as_read_on_new_window", "FALSE",
@@ -399,6 +412,8 @@ static PrefParam param[] = {
 	{"open_inbox_on_inc", "FALSE", &prefs_common.open_inbox_on_inc, P_BOOL},
 	{"open_inbox_on_startup", "FALSE", &prefs_common.open_inbox_on_startup,
 	 P_BOOL},
+	{"change_account_on_folder_selection", "TRUE",
+	 &prefs_common.change_account_on_folder_sel, P_BOOL},
 	{"immediate_execution", "TRUE", &prefs_common.immediate_exec, P_BOOL},
 
 #ifdef G_OS_WIN32
@@ -425,6 +440,12 @@ static PrefParam param[] = {
 
 	{"add_address_by_click", "FALSE", &prefs_common.add_address_by_click,
 	 P_BOOL},
+	{"enable_address_completion", "TRUE",
+	 &prefs_common.enable_address_completion, P_BOOL},
+	{"fullauto_completion_mode", "TRUE",
+	 &prefs_common.fullauto_completion_mode, P_BOOL},
+	{"always_add_address_only", "FALSE",
+	 &prefs_common.always_add_address_only, P_BOOL},
 
 	{"confirm_on_exit", "FALSE", &prefs_common.confirm_on_exit, P_BOOL},
 	{"clean_trash_on_exit", "FALSE", &prefs_common.clean_on_exit, P_BOOL},
@@ -500,8 +521,6 @@ void prefs_common_read_config(void)
 
 	prefs_common.online_mode = TRUE;
 
-	prefs_common_junk_filter_list_set();
-
 	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, COMMAND_HISTORY,
 			   NULL);
 	if ((fp = g_fopen(path, "rb")) == NULL) {
@@ -548,64 +567,8 @@ void prefs_common_write_config(void)
 	g_free(path);
 }
 
-static FilterRule *prefs_common_junk_filter_rule_create(gboolean is_manual)
-{
-	FilterRule *rule;
-	FilterCond *cond;
-	FilterAction *action;
-	GSList *cond_list = NULL, *action_list = NULL;
-
-	cond = filter_cond_new(FLT_COND_CMD_TEST, 0, 0, NULL,
-			       prefs_common.junk_classify_cmd);
-	cond_list = g_slist_append(NULL, cond);
-	if (prefs_common.delete_junk_on_recv && !is_manual) {
-		action = filter_action_new(FLT_ACTION_COPY,
-					   prefs_common.junk_folder);
-		action_list = g_slist_append(NULL, action);
-		action = filter_action_new(FLT_ACTION_DELETE, NULL);
-		action_list = g_slist_append(action_list, action);
-	} else {
-		action = filter_action_new(FLT_ACTION_MOVE,
-					   prefs_common.junk_folder);
-		action_list = g_slist_append(NULL, action);
-	}
-
-	if (prefs_common.mark_junk_as_read) {
-		action = filter_action_new(FLT_ACTION_MARK_READ, NULL);
-		action_list = g_slist_append(action_list, action);
-	}
-
-	if (is_manual)
-		rule = filter_rule_new(_("Junk mail filter (manual)"), FLT_OR,
-	 			       cond_list, action_list);
-	else
-		rule = filter_rule_new(_("Junk mail filter"), FLT_OR,
-				       cond_list, action_list);
-
-	return rule;
-}
-
 void prefs_common_junk_filter_list_set(void)
 {
-	FilterRule *rule;
-
-	if (prefs_common.junk_fltlist) {
-		filter_rule_list_free(prefs_common.junk_fltlist);
-		prefs_common.junk_fltlist = NULL;
-	}
-	if (prefs_common.manual_junk_fltlist) {
-		filter_rule_list_free(prefs_common.manual_junk_fltlist);
-		prefs_common.manual_junk_fltlist = NULL;
-	}
-
-	if (!prefs_common.junk_classify_cmd || !prefs_common.junk_folder)
-		return;
-
-	rule = prefs_common_junk_filter_rule_create(FALSE);
-	prefs_common.junk_fltlist = g_slist_append(NULL, rule);
-
-	rule = prefs_common_junk_filter_rule_create(TRUE);
-	prefs_common.manual_junk_fltlist = g_slist_append(NULL, rule);
 }
 
 void prefs_common_junk_folder_rename_path(const gchar *old_path,
@@ -637,6 +600,5 @@ void prefs_common_junk_folder_rename_path(const gchar *old_path,
 			    dest_path);
 		g_free(prefs_common.junk_folder);
 		prefs_common.junk_folder = dest_path;
-		prefs_common_junk_filter_list_set();
 	}
 }
