@@ -300,6 +300,8 @@ struct _PrefsCommon
 
 	/* Filtering */
 	GSList *fltlist;
+
+	/* deprecated: do not use */
 	GSList *junk_fltlist;
 	GSList *manual_junk_fltlist;
 
@@ -314,6 +316,15 @@ struct _PrefsCommon
 	gboolean reply_address_only;         /* Compose */
 	gboolean recipients_autoreg;         /* Send */
 	gboolean enable_address_completion;  /* Compose */
+	gboolean fullauto_completion_mode;   /* Compose */
+
+	gchar *user_agent_str;
+
+	gboolean change_account_on_folder_sel; /* Interface */
+	gboolean always_mark_read_on_show_msg; /* Interface */
+
+	gboolean always_add_address_only;    /* Compose */
+	gboolean show_send_dialog;           /* Send */
 };
 
 extern PrefsCommon prefs_common;
@@ -325,7 +336,9 @@ PrefParam *prefs_common_get_params	(void);
 void prefs_common_read_config		(void);
 void prefs_common_write_config		(void);
 
+/* deprecated */
 void prefs_common_junk_filter_list_set		(void);
+
 void prefs_common_junk_folder_rename_path	(const gchar	*old_path,
 						 const gchar	*new_path);
 
