@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkwidget.h>
+#include <gtk/gtkvbox.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtktreestore.h>
 #include <gtk/gtktreeview.h>
@@ -110,6 +111,7 @@ static GdkPixbuf *folderopen_pixbuf;
 static GdkPixbuf *foldernoselect_pixbuf;
 static GdkPixbuf *draft_pixbuf;
 static GdkPixbuf *trash_pixbuf;
+static GdkPixbuf *junk_pixbuf;
 static GdkPixbuf *virtual_pixbuf;
 
 static void folderview_set_columns	(FolderView	*folderview);
@@ -254,6 +256,7 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 	{N_("/_Move folder..."),	NULL, folderview_move_folder_cb, 0, NULL},
 	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/Empty _junk"),		NULL, folderview_empty_trash_cb, 0, NULL},
 	{N_("/Empty _trash"),		NULL, folderview_empty_trash_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Check for new messages"),
@@ -277,6 +280,7 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 	{N_("/_Move folder..."),	NULL, folderview_move_folder_cb, 0, NULL},
 	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/Empty _junk"),		NULL, folderview_empty_trash_cb, 0, NULL},
 	{N_("/Empty _trash"),		NULL, folderview_empty_trash_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Down_load"),		NULL, folderview_download_cb, 0, NULL},
@@ -321,6 +325,7 @@ static GtkItemFactoryEntry folderview_news_popup_entries[] =
 FolderView *folderview_create(void)
 {
 	FolderView *folderview;
+	GtkWidget *vbox;
 	GtkWidget *scrolledwin;
 	GtkWidget *treeview;
 	GtkTreeStore *store;
@@ -338,12 +343,15 @@ FolderView *folderview_create(void)
 	debug_print(_("Creating folder view...\n"));
 	folderview = g_new0(FolderView, 1);
 
+	vbox = gtk_vbox_new(FALSE, 1);
+
 	scrolledwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy
 		(GTK_SCROLLED_WINDOW(scrolledwin),
 		 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwin),
 					    GTK_SHADOW_IN);
+	gtk_box_pack_start(GTK_BOX(vbox), scrolledwin, TRUE, TRUE, 0);
 	gtk_widget_set_size_request(scrolledwin,
 				    prefs_common.folderview_width,
 				    prefs_common.folderview_height);
@@ -364,7 +372,9 @@ FolderView *folderview_create(void)
 	gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeview),
 					COL_FOLDER_NAME);
 	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(treeview), FALSE);
-	/* g_object_set(treeview, "fixed-height-mode", TRUE, NULL); */
+#if GTK_CHECK_VERSION(2, 12, 0)
+	g_object_set(treeview, "fixed-height-mode", TRUE, NULL);
+#endif
 
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
@@ -373,7 +383,7 @@ FolderView *folderview_create(void)
 
 	/* create folder icon + name column */
 	column = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_spacing(column, 2);
+	gtk_tree_view_column_set_spacing(column, 1);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width
 		(column, prefs_common.folder_col_folder);
@@ -519,6 +529,7 @@ FolderView *folderview_create(void)
 			 G_CALLBACK(folderview_drag_received_cb),
 			 folderview);
 
+	folderview->vbox         = vbox;
 	folderview->scrolledwin  = scrolledwin;
 	folderview->treeview     = treeview;
 	folderview->store        = store;
@@ -534,7 +545,7 @@ FolderView *folderview_create(void)
 
 	folderview_set_columns(folderview);
 
-	gtk_widget_show_all(scrolledwin);
+	gtk_widget_show_all(vbox);
 
 	folderview_list = g_list_append(folderview_list, folderview);
 
@@ -547,13 +558,14 @@ void folderview_init(FolderView *folderview)
 
 	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_INBOX, &inbox_pixbuf);
 	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_OUTBOX, &outbox_pixbuf);
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DIR_CLOSE, &folder_pixbuf);
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DIR_OPEN, &folderopen_pixbuf);
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DIR_NOSELECT,
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_CLOSE, &folder_pixbuf);
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_OPEN, &folderopen_pixbuf);
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_NOSELECT,
 			 &foldernoselect_pixbuf);
 	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DRAFT, &draft_pixbuf);
 	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_TRASH, &trash_pixbuf);
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_GROUP, &virtual_pixbuf);
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_SPAM_SMALL, &junk_pixbuf);
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_SEARCH, &virtual_pixbuf);
 }
 
 void folderview_reflect_prefs(FolderView *folderview)
@@ -567,6 +579,16 @@ void folderview_reflect_prefs(FolderView *folderview)
 	}
 }
 
+void folderview_add_sub_widget(FolderView *folderview, GtkWidget *widget)
+{
+	g_return_if_fail(folderview != NULL);
+	g_return_if_fail(widget != NULL);
+
+	debug_print("folderview_add_sub_widget: adding sub widget\n");
+
+	gtk_box_pack_start(GTK_BOX(folderview->vbox), widget, FALSE, FALSE, 0);
+}
+
 FolderView *folderview_get(void)
 {
 	return (FolderView *)folderview_list->data;
@@ -611,13 +633,13 @@ static void folderview_set_columns(FolderView *folderview)
 
 	column = gtk_tree_view_get_column(treeview, COL_NEW);
 	gtk_tree_view_column_set_visible
-		(column, prefs_common.display_folder_num_columns);
+		(column, prefs_common.folder_col_visible[COL_NEW]);
 	column = gtk_tree_view_get_column(treeview, COL_UNREAD);
 	gtk_tree_view_column_set_visible
-		(column, prefs_common.display_folder_num_columns);
+		(column, prefs_common.folder_col_visible[COL_UNREAD]);
 	column = gtk_tree_view_get_column(treeview, COL_TOTAL);
 	gtk_tree_view_column_set_visible
-		(column, prefs_common.display_folder_num_columns);
+		(column, prefs_common.folder_col_visible[COL_TOTAL]);
 }
 
 void folderview_select(FolderView *folderview, FolderItem *item)
@@ -912,6 +934,8 @@ static GtkWidget *label_window_create(const gchar *str)
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
 	manage_window_set_transient(GTK_WINDOW(window));
+	g_signal_connect(G_OBJECT(window), "delete_event", G_CALLBACK(gtk_true),
+			 NULL);
 
 	label = gtk_label_new(str);
 	gtk_container_add(GTK_CONTAINER(window), label);
@@ -1002,10 +1026,12 @@ gint folderview_check_new(Folder *folder)
 				break;
 		}
 		folderview_update_row(folderview, &iter);
-		if (prev_unread < item->unread)
-			n_updated += item->unread - prev_unread;
-		else if (prev_new < item->new)
-			n_updated += item->new - prev_new;
+		if (item->stype != F_TRASH && item->stype != F_JUNK) {
+			if (prev_unread < item->unread)
+				n_updated += item->unread - prev_unread;
+			else if (prev_new < item->new)
+				n_updated += item->new - prev_new;
+		}
 	}
 
 	gtk_widget_set_sensitive(folderview->treeview, TRUE);
@@ -1056,10 +1082,12 @@ gint folderview_check_new_item(FolderItem *item)
 	folderview_scan_tree_func(folder, item, NULL);
 	folder_item_scan(item);
 	folderview_update_row(folderview, &iter);
-	if (prev_unread < item->unread)
-		n_updated = item->unread - prev_unread;
-	else if (prev_new < item->new)
-		n_updated = item->new - prev_new;
+	if (item->stype != F_TRASH && item->stype != F_JUNK) {
+		if (prev_unread < item->unread)
+			n_updated = item->unread - prev_unread;
+		else if (prev_new < item->new)
+			n_updated = item->new - prev_new;
+	}
 
 	gtk_widget_set_sensitive(folderview->treeview, TRUE);
 	main_window_unlock(folderview->mainwin);
@@ -1246,15 +1274,12 @@ static void folderview_update_row(FolderView *folderview, GtkTreeIter *iter)
 				!strcmp2(item->name, DRAFT_DIR) ? _("Drafts") :
 				item->name);
 		break;
-#if 0
 	case F_JUNK:
-		pixbuf = folder_pixbuf;
-		open_pixbuf = folderopen_pixbuf;
+		pixbuf = open_pixbuf = junk_pixbuf;
 		name = g_strdup(FOLDER_IS_LOCAL(item->folder) &&
 				!strcmp2(item->name, JUNK_DIR) ? _("Junk") :
 				item->name);
 		break;
-#endif
 	case F_VIRTUAL:
 		pixbuf = open_pixbuf = virtual_pixbuf;
 		name = g_strdup(item->name);
@@ -1328,8 +1353,12 @@ static void folderview_update_row(FolderView *folderview, GtkTreeIter *iter)
 	}
 
 	if (item->stype == F_OUTBOX || item->stype == F_DRAFT ||
-	    item->stype == F_TRASH) {
+	    item->stype == F_TRASH || item->stype == F_JUNK) {
 		use_color = FALSE;
+		if (item->stype == F_JUNK) {
+			if ((item->unread > 0) || add_unread_mark)
+				weight = PANGO_WEIGHT_BOLD;
+		}
 	} else if (item->stype == F_QUEUE) {
 		/* highlight queue folder if there are any messages */
 		use_color = (item->total > 0);
@@ -1362,6 +1391,7 @@ static void folderview_update_row(FolderView *folderview, GtkTreeIter *iter)
 			   COL_FOREGROUND, foreground,
 			   COL_BOLD, weight,
 			   -1);
+	/* g_print("folderview_update_row: %s: %s\n", item->path, name); */
 	g_free(name);
 
 	item->updated = FALSE;
@@ -1527,7 +1557,7 @@ void folderview_move_folder(FolderView *folderview)
 	g_return_if_fail(item->folder != NULL);
 
 	if (!item->path) return;
-	if (item->stype != F_NORMAL) return;
+	if (item->stype != F_NORMAL || item->stype != F_VIRTUAL) return;
 
 	if (item->folder->klass->move_folder)
 		folderview_move_folder_cb(folderview, 0, NULL);
@@ -1621,6 +1651,7 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 	gboolean rename_folder   = FALSE;
 	gboolean move_folder     = FALSE;
 	gboolean delete_folder   = FALSE;
+	gboolean empty_junk      = FALSE;
 	gboolean empty_trash     = FALSE;
 	gboolean download_msg    = FALSE;
 	gboolean update_tree     = FALSE;
@@ -1671,6 +1702,9 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 			} else if (item->stype == F_TRASH) {
 				if (item->total > 0)
 					empty_trash = TRUE;
+			} else if (item->stype == F_JUNK) {
+				if (item->total > 0)
+					empty_junk = TRUE;
 			} else if (item->stype == F_QUEUE) {
 				if (item->total > 0)
 					send_queue = TRUE;
@@ -1681,7 +1715,7 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 		}
 		if (item->stype == F_VIRTUAL) {
 			new_folder = FALSE;
-			rename_folder = delete_folder = TRUE;
+			move_folder = rename_folder = delete_folder = TRUE;
 		}
 		if (FOLDER_TYPE(folder) == F_IMAP ||
 		    FOLDER_TYPE(folder) == F_NEWS) {
@@ -1702,6 +1736,9 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 			if (item->stype == F_TRASH) {
 				if (item->total > 0)
 					empty_trash = TRUE;
+			} else if (item->stype == F_JUNK) {
+				if (item->total > 0)
+					empty_junk = TRUE;
 			}
 		}
 	}
@@ -1726,6 +1763,28 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 	}							\
 }
 
+#define SET_VISIBILITY2(factory, name, visible)			\
+{								\
+	GtkWidget *widget;					\
+	widget = gtk_item_factory_get_item(factory, name);	\
+	if (widget) {						\
+		GList *child;					\
+		GtkWidget *sep = NULL;				\
+								\
+		child = g_list_find				\
+			(GTK_MENU_SHELL(popup)->children, widget); \
+		if (child && child->next)			\
+			sep = GTK_WIDGET(child->next->data);	\
+		if (visible) {					\
+			gtk_widget_show(widget);		\
+			gtk_widget_show(sep);			\
+		} else {					\
+			gtk_widget_hide(widget);		\
+			gtk_widget_hide(sep);			\
+		}						\
+	}							\
+}
+
 	if (FOLDER_IS_LOCAL(folder)) {
 		popup = folderview->mail_popup;
 		ifactory = folderview->mail_factory;
@@ -1744,6 +1803,7 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 	SET_SENS(ifactory, "/Rename folder...", rename_folder);
 	SET_SENS(ifactory, "/Move folder...", move_folder);
 	SET_SENS(ifactory, "/Delete folder", delete_folder);
+	SET_SENS(ifactory, "/Empty junk", empty_junk);
 	SET_SENS(ifactory, "/Empty trash", empty_trash);
 	SET_SENS(ifactory, "/Download", download_msg);
 	SET_SENS(ifactory, "/Check for new messages", update_tree);
@@ -1766,9 +1826,37 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 			       item->stype == F_VIRTUAL);
 	}
 
-	SET_VISIBILITY(ifactory, "/Mark all read", item->stype != F_QUEUE);
-	SET_VISIBILITY(ifactory, "/Send queued messages",
-		       item->stype == F_QUEUE);
+	if (item->stype == F_JUNK) {
+		SET_VISIBILITY(ifactory, "/Empty junk", TRUE);
+		SET_VISIBILITY2(ifactory, "/Empty trash", TRUE);
+		SET_VISIBILITY(ifactory, "/Empty trash", FALSE);
+	} else {
+		SET_VISIBILITY(ifactory, "/Empty junk", FALSE);
+		SET_VISIBILITY2(ifactory, "/Empty trash",
+				item->stype == F_TRASH);
+	}
+
+	SET_VISIBILITY(ifactory, "/Check for new messages",
+		       item->parent == NULL);
+	SET_VISIBILITY(ifactory, "/Rebuild folder tree", item->parent == NULL);
+	SET_VISIBILITY(ifactory, "/Update summary", item->parent != NULL);
+
+	if (FOLDER_TYPE(folder) == F_NEWS) {
+		SET_VISIBILITY2(ifactory, "/Mark all read",
+				item->parent != NULL && item->stype != F_QUEUE);
+	} else {
+		SET_VISIBILITY(ifactory, "/Mark all read",
+			       item->parent != NULL && item->stype != F_QUEUE);
+		if (item->parent != NULL) {
+			SET_VISIBILITY2(ifactory, "/Send queued messages", TRUE);
+			SET_VISIBILITY(ifactory, "/Send queued messages",
+				       item->stype == F_QUEUE);
+		} else {
+			SET_VISIBILITY2(ifactory, "/Send queued messages",
+					item->stype == F_QUEUE);
+		}
+	}
+
 	SET_VISIBILITY(ifactory, "/Search messages...",
 		       item->stype != F_VIRTUAL);
 	SET_VISIBILITY(ifactory, "/Edit search condition...",
@@ -1776,6 +1864,7 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 
 #undef SET_SENS
 #undef SET_VISIBILITY
+#undef SET_VISIBILITY2
 
 	syl_plugin_signal_emit("folderview-menu-popup", ifactory);
 
@@ -1841,7 +1930,10 @@ static gboolean folderview_button_released(GtkWidget *treeview,
 static gboolean folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				       FolderView *folderview)
 {
+	GtkTreeView *treeview = GTK_TREE_VIEW(widget);
 	GtkTreePath *opened = NULL, *selected = NULL;
+	GtkAdjustment *adj;
+	gboolean moved;
 
 	if (!event) return FALSE;
 
@@ -1857,7 +1949,6 @@ static gboolean folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 						  folderview->selected);
 		}
 		return TRUE;
-		break;
 	case GDK_space:
 	case GDK_KP_Space:
 		if (folderview->selected) {
@@ -1879,6 +1970,52 @@ static gboolean folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 			return TRUE;
 		}
 		break;
+	case GDK_Left:
+	case GDK_KP_Left:
+		if ((event->state &
+		     (GDK_SHIFT_MASK|GDK_MOD1_MASK|GDK_CONTROL_MASK)) != 0)
+			return FALSE;
+		adj = gtk_scrolled_window_get_hadjustment
+			(GTK_SCROLLED_WINDOW(folderview->scrolledwin));
+		if (adj->lower < adj->value)
+			return FALSE;
+		if (folderview->selected) {
+			selected = gtk_tree_row_reference_get_path
+				(folderview->selected);
+			if (selected) {
+				if (gtk_tree_view_row_expanded(treeview, selected)) {
+					gtk_tree_view_collapse_row(treeview, selected);
+					gtk_tree_path_free(selected);
+					return TRUE;
+				}
+				gtk_tree_path_free(selected);
+			}
+		}
+		g_signal_emit_by_name(G_OBJECT(treeview),
+				      "select-cursor-parent", &moved);
+		return TRUE;
+	case GDK_Right:
+	case GDK_KP_Right:
+		if ((event->state &
+		     (GDK_SHIFT_MASK|GDK_MOD1_MASK|GDK_CONTROL_MASK)) != 0)
+			return FALSE;
+		adj = gtk_scrolled_window_get_hadjustment
+			(GTK_SCROLLED_WINDOW(folderview->scrolledwin));
+		if (adj->upper - adj->page_size > adj->value)
+			return FALSE;
+		if (folderview->selected) {
+			selected = gtk_tree_row_reference_get_path
+				(folderview->selected);
+			if (selected) {
+				if (!gtk_tree_view_row_expanded(treeview, selected)) {
+					gtk_tree_view_expand_row(treeview, selected, FALSE);
+					gtk_tree_path_free(selected);
+					return TRUE;
+				}
+				gtk_tree_path_free(selected);
+			}
+		}
+		break;
 	default:
 		break;
 	}
@@ -1890,7 +2027,9 @@ static gboolean folderview_focus_idle_func(gpointer data)
 {
 	FolderView *folderview = (FolderView *)data;
 
+	gdk_threads_enter();
 	GTK_WIDGET_SET_FLAGS(folderview->treeview, GTK_CAN_FOCUS);
+	gdk_threads_leave();
 
 	return FALSE;
 }
@@ -1975,6 +2114,20 @@ static void folderview_selection_changed(GtkTreeSelection *selection,
 	gtk_tree_path_free(path);
 
 	folderview->selection_locked = FALSE;
+
+	if (prefs_common.change_account_on_folder_sel) {
+		PrefsAccount *account;
+
+		account = account_find_from_item_property(item);
+		if (!account && item->folder)
+			account = item->folder->account;
+		if (!account)
+			account = account_get_default();
+		if (account && account != cur_account) {
+			cur_account = account;
+			main_window_change_cur_account();
+		}
+	}
 }
 
 static void folderview_row_expanded(GtkTreeView *treeview, GtkTreeIter *iter,
@@ -2287,7 +2440,7 @@ static void folderview_new_folder_cb(FolderView *folderview, guint action,
 #ifdef G_OS_WIN32
 	p = strpbrk(new_folder, "\\/:*?\"<>|");
 	if ((p && FOLDER_TYPE(item->folder) != F_IMAP) || (p && *p != '/') ||
-	    (p && p == '/' &&
+	    (p && *p == '/' &&
 	     FOLDER_TYPE(item->folder) == F_IMAP && *(p + 1) != '\0')) {
 		alertpanel_error(_("`%c' can't be included in folder name."),
 				 *p);
@@ -2605,21 +2758,29 @@ static void folderview_empty_trash_cb(FolderView *folderview, guint action,
 
 	folder = item->folder;
 
-	if (folder->trash != item) return;
-	if (item->stype != F_TRASH) return;
+	if (item->stype != F_TRASH && item->stype != F_JUNK) return;
 
 	sel_path = gtk_tree_row_reference_get_path(folderview->selected);
 
-	if (alertpanel(_("Empty trash"),
-		       _("Delete all messages in the trash folder?"),
-		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT) {
-		gtk_tree_path_free(sel_path);
-		return;
+	if (item->stype == F_TRASH) {
+		if (alertpanel(_("Empty trash"),
+			       _("Delete all messages in the trash folder?"),
+			       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT) {
+			gtk_tree_path_free(sel_path);
+			return;
+		}
+	} else {
+		if (alertpanel(_("Empty junk"),
+			       _("Delete all messages in the junk folder?"),
+			       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT) {
+			gtk_tree_path_free(sel_path);
+			return;
+		}
 	}
 
-	procmsg_empty_trash(folder->trash);
+	procmsg_empty_trash(item);
 	statusbar_pop_all();
-	folderview_update_item(folder->trash, TRUE);
+	folderview_update_item(item, TRUE);
 	trayicon_set_tooltip(NULL);
 	trayicon_set_notify(FALSE);
 
@@ -3001,6 +3162,9 @@ static gint auto_expand_timeout(gpointer data)
 	FolderView *folderview = data;
 	GtkTreeView *treeview = GTK_TREE_VIEW(folderview->treeview);
 	GtkTreePath *path = NULL;
+	gint ret;
+
+	gdk_threads_enter();
 
 	gtk_tree_view_get_drag_dest_row(treeview, &path, NULL);
 
@@ -3009,9 +3173,13 @@ static gint auto_expand_timeout(gpointer data)
 		gtk_tree_path_free(path);
 		folderview->expand_timeout = 0;
 
-		return FALSE;
+		ret = FALSE;
 	} else
-		return TRUE;
+		ret = TRUE;
+
+	gdk_threads_leave();
+
+	return ret;
 }
 
 static void remove_auto_expand_timeout(FolderView *folderview)
@@ -3026,9 +3194,13 @@ static gint auto_scroll_timeout(gpointer data)
 {
 	FolderView *folderview = data;
 
+	gdk_threads_enter();
+
 	gtkut_tree_view_vertical_autoscroll
 		(GTK_TREE_VIEW(folderview->treeview));
 
+	gdk_threads_leave();
+
 	return TRUE;
 }
 
@@ -3065,6 +3237,9 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 	} else
 		remove_auto_expand_timeout(folderview);
 
+	if (summary_is_locked(folderview->summaryview))
+		acceptable = FALSE;
+
 	gtk_tree_view_get_drag_dest_row(GTK_TREE_VIEW(widget),
 					&prev_path, NULL);
 	if (!path || (prev_path && gtk_tree_path_compare(path, prev_path) != 0))
