@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "plugin.h"
 #include "utils.h"
 #include "folder.h"
+#include "plugin-marshal.h"
 
 G_DEFINE_TYPE(SylPlugin, syl_plugin, G_TYPE_OBJECT);
 
@@ -31,11 +32,15 @@ enum {
 	PLUGIN_LOAD,
 	PLUGIN_UNLOAD,
 	FOLDERVIEW_MENU_POPUP,
+	SUMMARYVIEW_MENU_POPUP,
 	COMPOSE_CREATED,
 	COMPOSE_DESTROY,
+	TEXTVIEW_MENU_POPUP,
 	LAST_SIGNAL
 };
 
+#define GETFUNC(sym)	{ func = syl_plugin_lookup_symbol(sym); }
+
 #define SAFE_CALL(func_ptr)		{ if (func_ptr) func_ptr(); }
 #define SAFE_CALL_RET(func_ptr)		(func_ptr ? func_ptr() : NULL)
 #define SAFE_CALL_RET_VAL(func_ptr, retval) \
@@ -55,10 +60,26 @@ enum {
 				{ if (func_ptr) func_ptr(arg1, arg2, arg3); }
 #define SAFE_CALL_ARG3_RET(func_ptr, arg1, arg2, arg3) \
 				(func_ptr ? func_ptr(arg1, arg2, arg3) : NULL)
+#define SAFE_CALL_ARG3_RET_VAL(func_ptr, arg1, arg2, arg3, retval) \
+				(func_ptr ? func_ptr(arg1, arg2, arg3) : retval)
 #define SAFE_CALL_ARG4(func_ptr, arg1, arg2, arg3, arg4) \
 				{ if (func_ptr) func_ptr(arg1, arg2, arg3); }
 #define SAFE_CALL_ARG4_RET(func_ptr, arg1, arg2, arg3, arg4) \
 				(func_ptr ? func_ptr(arg1, arg2, arg3, arg4) : NULL)
+#define SAFE_CALL_ARG4_RET_VAL(func_ptr, arg1, arg2, arg3, arg4, retval) \
+				(func_ptr ? func_ptr(arg1, arg2, arg3, arg4) : retval)
+
+#define CALL_VOID_POINTER(getobj, sym)		\
+{						\
+	void (*func)(gpointer);			\
+	gpointer obj;				\
+	obj = getobj();				\
+	if (obj) {				\
+		GETFUNC(sym);			\
+		SAFE_CALL_ARG1(func, obj);	\
+	}					\
+}
+
 
 static guint plugin_signals[LAST_SIGNAL] = { 0 };
 
@@ -105,6 +126,17 @@ static void syl_plugin_class_init(SylPluginClass *klass)
 			     G_TYPE_NONE,
 			     1,
 			     G_TYPE_POINTER);
+	plugin_signals[SUMMARYVIEW_MENU_POPUP] =
+		g_signal_new("summaryview-menu-popup",
+			     G_TYPE_FROM_CLASS(gobject_class),
+			     G_SIGNAL_RUN_FIRST,
+			     G_STRUCT_OFFSET(SylPluginClass,
+					     summaryview_menu_popup),
+			     NULL, NULL,
+			     g_cclosure_marshal_VOID__POINTER,
+			     G_TYPE_NONE,
+			     1,
+			     G_TYPE_POINTER);
 	plugin_signals[COMPOSE_CREATED] =
 		g_signal_new("compose-created",
 			     G_TYPE_FROM_CLASS(gobject_class),
@@ -125,6 +157,21 @@ static void syl_plugin_class_init(SylPluginClass *klass)
 			     G_TYPE_NONE,
 			     1,
 			     G_TYPE_POINTER);
+	plugin_signals[TEXTVIEW_MENU_POPUP] =
+		g_signal_new("textview-menu-popup",
+			     G_TYPE_FROM_CLASS(gobject_class),
+			     G_SIGNAL_RUN_FIRST,
+			     G_STRUCT_OFFSET(SylPluginClass,
+					     textview_menu_popup),
+			     NULL, NULL,
+			     syl_plugin_marshal_VOID__POINTER_POINTER_STRING_STRING_POINTER,
+			     G_TYPE_NONE,
+			     5,
+			     G_TYPE_POINTER,
+			     G_TYPE_POINTER,
+			     G_TYPE_STRING,
+			     G_TYPE_STRING,
+			     G_TYPE_POINTER);
 }
 
 void syl_plugin_signal_connect(const gchar *name, GCallback callback,
@@ -362,26 +409,14 @@ const gchar *syl_plugin_get_prog_version(void)
 
 void syl_plugin_main_window_lock(void)
 {
-	void (*func)(gpointer);
-	gpointer mainwin;
-
-	mainwin = syl_plugin_main_window_get();
-	if (mainwin) {
-		func = syl_plugin_lookup_symbol("main_window_lock");
-		SAFE_CALL_ARG1(func, mainwin);
-	}
+	CALL_VOID_POINTER(syl_plugin_main_window_get,
+			  "main_window_lock");
 }
 
 void syl_plugin_main_window_unlock(void)
 {
-	void (*func)(gpointer);
-	gpointer mainwin;
-
-	mainwin = syl_plugin_main_window_get();
-	if (mainwin) {
-		func = syl_plugin_lookup_symbol("main_window_unlock");
-		SAFE_CALL_ARG1(func, mainwin);
-	}
+	CALL_VOID_POINTER(syl_plugin_main_window_get,
+			  "main_window_unlock");
 }
 
 gpointer syl_plugin_main_window_get(void)
@@ -431,6 +466,8 @@ static GtkItemFactory *get_item_factory(const gchar *path)
 		ifactory = syl_plugin_lookup_symbol("folderview_imap_popup_factory");
 	else if (strncmp(path, "<NewsFolder>", 12) == 0)
 		ifactory = syl_plugin_lookup_symbol("folderview_news_popup_factory");
+	else if (strncmp(path, "<SummaryView>", 13) == 0)
+		ifactory = syl_plugin_lookup_symbol("summaryview_popup_factory");
 	else
 		ifactory = syl_plugin_lookup_symbol("main_window_menu_factory");
 
@@ -542,10 +579,45 @@ void syl_plugin_menu_set_active(const gchar *path, gboolean is_active)
 
 gpointer syl_plugin_folderview_get(void)
 {
-	gpointer sym;
+	gpointer (*func)(void);
+	GETFUNC("folderview_get");
+	return SAFE_CALL_RET(func);
+}
 
-	sym = syl_plugin_lookup_symbol("folderview");
-	return sym;
+void syl_plugin_folderview_add_sub_widget(GtkWidget *widget)
+{
+	void (*func)(gpointer, GtkWidget *);
+	gpointer folderview;
+
+	folderview = syl_plugin_folderview_get();
+	if (folderview) {
+		GETFUNC("folderview_add_sub_widget");
+		SAFE_CALL_ARG2(func, folderview, widget);
+	}
+}
+
+void syl_plugin_folderview_select(FolderItem *item)
+{
+	void (*func)(gpointer, FolderItem *);
+	gpointer folderview;
+
+	folderview = syl_plugin_folderview_get();
+	if (folderview) {
+		GETFUNC("folderview_select");
+		SAFE_CALL_ARG2(func, folderview, item);
+	}
+}
+
+void syl_plugin_folderview_unselect(void)
+{
+	CALL_VOID_POINTER(syl_plugin_folderview_get,
+			  "folderview_unselect");
+}
+
+void syl_plugin_folderview_select_next_unread(void)
+{
+	CALL_VOID_POINTER(syl_plugin_folderview_get,
+			  "folderview_select_next_unread");
 }
 
 FolderItem *syl_plugin_folderview_get_selected_item(void)
@@ -555,13 +627,63 @@ FolderItem *syl_plugin_folderview_get_selected_item(void)
 
 	folderview = syl_plugin_folderview_get();
 	if (folderview) {
-		func = syl_plugin_lookup_symbol("folderview_get_selected_item");
+		GETFUNC("folderview_get_selected_item");
 		return SAFE_CALL_ARG1_RET(func, folderview);
 	}
 
 	return NULL;
 }
 
+gint syl_plugin_folderview_check_new(Folder *folder)
+{
+	gint (*func)(Folder *);
+	GETFUNC("folderview_check_new");
+	return SAFE_CALL_ARG1_RET_VAL(func, folder, FALSE);
+}
+
+gint syl_plugin_folderview_check_new_item(FolderItem *item)
+{
+	gint (*func)(FolderItem *);
+	GETFUNC("folderview_check_new_item");
+	return SAFE_CALL_ARG1_RET_VAL(func, item, FALSE);
+}
+
+gint syl_plugin_folderview_check_new_all(void)
+{
+	gint (*func)(void);
+	GETFUNC("folderview_check_new_all");
+	return SAFE_CALL_RET_VAL(func, FALSE);
+}
+
+void syl_plugin_folderview_update_item(FolderItem *item,
+				       gboolean update_summary)
+{
+	void (*func)(FolderItem *, gboolean);
+	GETFUNC("folderview_update_item");
+	SAFE_CALL_ARG2(func, item, update_summary);
+}
+
+void syl_plugin_folderview_update_item_foreach(GHashTable *table,
+					       gboolean update_summary)
+{
+	void (*func)(GHashTable *, gboolean);
+	GETFUNC("folderview_update_item_foreach");
+	SAFE_CALL_ARG2(func, table, update_summary);
+}
+
+void syl_plugin_folderview_update_all_updated(gboolean update_summary)
+{
+	void (*func)(gboolean);
+	GETFUNC("folderview_update_all_updated");
+	SAFE_CALL_ARG1(func, update_summary);
+}
+
+void syl_plugin_folderview_check_new_selected(void)
+{
+	CALL_VOID_POINTER(syl_plugin_folderview_get,
+			  "folderview_check_new_selected");
+}
+
 gpointer syl_plugin_summary_view_get(void)
 {
 	gpointer sym;
@@ -570,7 +692,7 @@ gpointer syl_plugin_summary_view_get(void)
 	return sym;
 }
 
-void syl_plugin_sumary_select_by_msgnum(guint msgnum)
+void syl_plugin_summary_select_by_msgnum(guint msgnum)
 {
 	void (*func)(gpointer, guint);
 	gpointer summary;
@@ -612,28 +734,22 @@ void syl_plugin_open_message(const gchar *folder_id, guint msgnum)
 	}
 }
 
+void syl_plugin_summary_show_queued_msgs(void)
+{
+	CALL_VOID_POINTER(syl_plugin_summary_view_get,
+			  "summary_show_queued_msgs");
+}
+
 void syl_plugin_summary_lock(void)
 {
-	void (*func)(gpointer);
-	gpointer summary;
-
-	summary = syl_plugin_summary_view_get();
-	if (summary) {
-		func = syl_plugin_lookup_symbol("summary_lock");
-		SAFE_CALL_ARG1(func, summary);
-	}
+	CALL_VOID_POINTER(syl_plugin_summary_view_get,
+			  "summary_lock");
 }
 
 void syl_plugin_summary_unlock(void)
 {
-	void (*func)(gpointer);
-	gpointer summary;
-
-	summary = syl_plugin_summary_view_get();
-	if (summary) {
-		func = syl_plugin_lookup_symbol("summary_unlock");
-		SAFE_CALL_ARG1(func, summary);
-	}
+	CALL_VOID_POINTER(syl_plugin_summary_view_get,
+			  "summary_unlock");
 }
 
 gboolean syl_plugin_summary_is_locked(void)
@@ -643,13 +759,151 @@ gboolean syl_plugin_summary_is_locked(void)
 
 	summary = syl_plugin_summary_view_get();
 	if (summary) {
-		func = syl_plugin_lookup_symbol("summary_is_locked");
+		GETFUNC("summary_is_locked");
 		return SAFE_CALL_ARG1_RET_VAL(func, summary, FALSE);
 	}
 
 	return FALSE;
 }
 
+gboolean syl_plugin_summary_is_read_locked(void)
+{
+	gboolean (*func)(gpointer);
+	gpointer summary;
+
+	summary = syl_plugin_summary_view_get();
+	if (summary) {
+		GETFUNC("summary_is_read_locked");
+		return SAFE_CALL_ARG1_RET_VAL(func, summary, FALSE);
+	}
+
+	return FALSE;
+}
+
+void syl_plugin_summary_write_lock(void)
+{
+	CALL_VOID_POINTER(syl_plugin_summary_view_get,
+			  "summary_write_lock");
+}
+
+void syl_plugin_summary_write_unlock(void)
+{
+	CALL_VOID_POINTER(syl_plugin_summary_view_get,
+			  "summary_write_unlock");
+}
+
+gboolean syl_plugin_summary_is_write_locked(void)
+{
+	gboolean (*func)(gpointer);
+	gpointer summary;
+
+	summary = syl_plugin_summary_view_get();
+	if (summary) {
+		GETFUNC("summary_is_write_locked");
+		return SAFE_CALL_ARG1_RET_VAL(func, summary, FALSE);
+	}
+
+	return FALSE;
+}
+
+FolderItem *syl_plugin_summary_get_current_folder(void)
+{
+	FolderItem * (*func)(gpointer);
+	gpointer summary;
+
+	summary = syl_plugin_summary_view_get();
+	if (summary) {
+		GETFUNC("summary_get_current_folder");
+		return SAFE_CALL_ARG1_RET(func, summary);
+	}
+
+	return NULL;
+}
+
+gint syl_plugin_summary_get_selection_type(void)
+{
+	gint (*func)(gpointer);
+	gpointer summary;
+
+	summary = syl_plugin_summary_view_get();
+	if (summary) {
+		GETFUNC("summary_get_selection_type");
+		return SAFE_CALL_ARG1_RET_VAL(func, summary, 0);
+	}
+
+	return 0;
+}
+
+GSList *syl_plugin_summary_get_selected_msg_list(void)
+{
+	GSList * (*func)(gpointer);
+	gpointer summary;
+
+	summary = syl_plugin_summary_view_get();
+	if (summary) {
+		GETFUNC("summary_get_selected_msg_list");
+		return SAFE_CALL_ARG1_RET(func, summary);
+	}
+
+	return NULL;
+}
+
+GSList *syl_plugin_summary_get_msg_list(void)
+{
+	GSList * (*func)(gpointer);
+	gpointer summary;
+
+	summary = syl_plugin_summary_view_get();
+	if (summary) {
+		GETFUNC("summary_get_msg_list");
+		return SAFE_CALL_ARG1_RET(func, summary);
+	}
+
+	return NULL;
+}
+
+void syl_plugin_summary_redisplay_msg(void)
+{
+	CALL_VOID_POINTER(syl_plugin_summary_view_get,
+			  "summary_redisplay_msg");
+}
+
+void syl_plugin_summary_open_msg(void)
+{
+	CALL_VOID_POINTER(syl_plugin_summary_view_get,
+			  "summary_open_msg");
+}
+
+void syl_plugin_summary_view_source(void)
+{
+	CALL_VOID_POINTER(syl_plugin_summary_view_get,
+			  "summary_view_source");
+}
+
+void syl_plugin_summary_reedit(void)
+{
+	CALL_VOID_POINTER(syl_plugin_summary_view_get,
+			  "summary_reedit");
+}
+
+void syl_plugin_summary_update_selected_rows(void)
+{
+	CALL_VOID_POINTER(syl_plugin_summary_view_get,
+			  "summary_update_selected_rows");
+}
+
+void syl_plugin_summary_update_by_msgnum(guint msgnum)
+{
+	void (*func)(gpointer, guint);
+	gpointer summary;
+
+	summary = syl_plugin_summary_view_get();
+	if (summary) {
+		func = syl_plugin_lookup_symbol("summary_update_by_msgnum");
+		SAFE_CALL_ARG2(func, summary, msgnum);
+	}
+}
+
 gpointer syl_plugin_messageview_create_with_new_window(void)
 {
 	gpointer (*func)(void);
@@ -818,3 +1072,173 @@ void syl_plugin_inc_unlock(void)
 	func = syl_plugin_lookup_symbol("inc_unlock");
 	SAFE_CALL(func);
 }
+
+void syl_plugin_update_check(gboolean show_dialog_always)
+{
+	void (*func)(gboolean);
+
+	func = syl_plugin_lookup_symbol("update_check");
+	SAFE_CALL_ARG1(func, show_dialog_always);
+}
+
+void syl_plugin_update_check_set_check_url(const gchar *url)
+{
+	void (*func)(const gchar *);
+
+	func = syl_plugin_lookup_symbol("update_check_set_check_url");
+	SAFE_CALL_ARG1(func, url);
+}
+
+const gchar *syl_plugin_update_check_get_check_url(void)
+{
+	const gchar * (*func)(void);
+
+	func = syl_plugin_lookup_symbol("update_check_get_check_url");
+	return SAFE_CALL_RET(func);
+}
+
+void syl_plugin_update_check_set_download_url(const gchar *url)
+{
+	void (*func)(const gchar *);
+
+	func = syl_plugin_lookup_symbol("update_check_set_download_url");
+	SAFE_CALL_ARG1(func, url);
+}
+
+const gchar *syl_plugin_update_check_get_download_url(void)
+{
+	const gchar * (*func)(void);
+
+	func = syl_plugin_lookup_symbol("update_check_get_download_url");
+	return SAFE_CALL_RET(func);
+}
+
+void syl_plugin_update_check_set_jump_url(const gchar *url)
+{
+	void (*func)(const gchar *);
+
+	func = syl_plugin_lookup_symbol("update_check_set_jump_url");
+	SAFE_CALL_ARG1(func, url);
+}
+
+const gchar *syl_plugin_update_check_get_jump_url(void)
+{
+	const gchar * (*func)(void);
+
+	func = syl_plugin_lookup_symbol("update_check_get_jump_url");
+	return SAFE_CALL_RET(func);
+}
+
+void syl_plugin_update_check_set_check_plugin_url(const gchar *url)
+{
+	void (*func)(const gchar *);
+
+	func = syl_plugin_lookup_symbol("update_check_set_check_plugin_url");
+	SAFE_CALL_ARG1(func, url);
+}
+
+const gchar *syl_plugin_update_check_get_check_plugin_url(void)
+{
+	const gchar * (*func)(void);
+
+	func = syl_plugin_lookup_symbol("update_check_get_check_plugin_url");
+	return SAFE_CALL_RET(func);
+}
+
+void syl_plugin_update_check_set_jump_plugin_url(const gchar *url)
+{
+	void (*func)(const gchar *);
+
+	func = syl_plugin_lookup_symbol("update_check_set_jump_plugin_url");
+	SAFE_CALL_ARG1(func, url);
+}
+
+const gchar *syl_plugin_update_check_get_jump_plugin_url(void)
+{
+	const gchar * (*func)(void);
+
+	func = syl_plugin_lookup_symbol("update_check_get_jump_plugin_url");
+	return SAFE_CALL_RET(func);
+}
+
+gint syl_plugin_alertpanel_full(const gchar *title, const gchar *message,
+				gint type, gint default_value,
+				gboolean can_disable,
+				const gchar *btn1_label,
+				const gchar *btn2_label,
+				const gchar *btn3_label)
+{
+	gint (*func)(const gchar *, const gchar *, gint, gint, gboolean,
+			const gchar *, const gchar *, const gchar *);
+
+	GETFUNC("alertpanel_full");
+	return func ? func(title, message, type, default_value, can_disable,
+			   btn1_label, btn2_label, btn3_label) : -1;
+}
+
+gint syl_plugin_alertpanel(const gchar *title, const gchar *message,
+			   const gchar *btn1_label,
+			   const gchar *btn2_label,
+			   const gchar *btn3_label)
+{
+	gint (*func)(const gchar *, const gchar *,
+			const gchar *, const gchar *, const gchar *);
+
+	GETFUNC("alertpanel");
+	return func ? func(title, message, btn1_label, btn2_label, btn3_label)
+		: -1;
+}
+
+void syl_plugin_alertpanel_message(const gchar *title, const gchar *message,
+				   gint type)
+{
+	void (*func)(const gchar *, const gchar *, gint);
+
+	GETFUNC("alertpanel_message");
+	SAFE_CALL_ARG3(func, title, message, type);
+}
+
+gint syl_plugin_alertpanel_message_with_disable(const gchar *title,
+						const gchar *message,
+						gint type)
+{
+	gint (*func)(const gchar *, const gchar *, gint);
+
+	GETFUNC("alertpanel_message_with_disable");
+	return SAFE_CALL_ARG3_RET_VAL(func, title, message, type, 0);
+}
+
+gint syl_plugin_send_message(const gchar *file, PrefsAccount *ac,
+			     GSList *to_list)
+{
+	gint (*func)(const gchar *, PrefsAccount *, GSList *);
+
+	GETFUNC("send_message");
+	return SAFE_CALL_ARG3_RET_VAL(func, file, ac, to_list, -1);
+}
+
+gint syl_plugin_send_message_queue_all(FolderItem *queue, gboolean save_msgs,
+				       gboolean filter_msgs)
+{
+	gint (*func)(FolderItem *, gboolean, gboolean);
+
+	GETFUNC("send_message_queue_all");
+	return SAFE_CALL_ARG3_RET_VAL(func, queue, save_msgs, filter_msgs, -1);
+}
+
+gint syl_plugin_send_message_set_reply_flag(const gchar *reply_target,
+					    const gchar *msgid)
+{
+	gint (*func)(const gchar *, const gchar *);
+
+	GETFUNC("send_message_set_reply_flag");
+	return SAFE_CALL_ARG2_RET_VAL(func, reply_target, msgid, -1);
+}
+
+gint syl_plugin_send_message_set_forward_flags(const gchar *forward_targets)
+{
+	gint (*func)(const gchar *);
+
+	GETFUNC("send_message_set_forward_flags");
+	return SAFE_CALL_ARG1_RET_VAL(func, forward_targets, -1);
+}
