@@ -129,6 +129,8 @@ static void prefs_filter_edit_cancel		(void);
 
 static void prefs_filter_cond_activated_cb	(GtkWidget	*widget,
 						 gpointer	 data);
+static void prefs_filter_match_activated_cb	(GtkWidget	*widget,
+						 gpointer	 data);
 static void prefs_filter_action_activated_cb	(GtkWidget	*widget,
 						 gpointer	 data);
 
@@ -271,6 +273,7 @@ static void prefs_filter_edit_create(void)
 	MENUITEM_ADD(menu, menuitem,
 		     _("If all of the following conditions match"), FLT_AND);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(bool_op_optmenu), menu);
+	gtk_option_menu_set_history(GTK_OPTION_MENU(bool_op_optmenu), FLT_AND);
 
 	cond_scrolled_win = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(cond_scrolled_win);
@@ -367,7 +370,7 @@ static void prefs_filter_edit_clear(void)
 static void prefs_filter_edit_rule_to_dialog(FilterRule *rule,
 					     const gchar *default_name)
 {
-	gint index = 0;
+	gint index = 1;
 	static gint count = 1;
 
 	if (rule && rule->name)
@@ -388,7 +391,7 @@ static void prefs_filter_edit_rule_to_dialog(FilterRule *rule,
 			(GTK_OPTION_MENU(rule_edit_window.bool_op_optmenu),
 			 GINT_TO_POINTER(rule->bool_op), NULL);
 		if (index < 0)
-			index = 0;
+			index = 1;
 	}
 	if (index >= 0) {
 		gtk_option_menu_set_history
@@ -472,6 +475,8 @@ CondHBox *prefs_filter_edit_cond_hbox_create(FilterCondEdit *cond_edit)
 	GtkWidget *add_btn;
 	GtkWidget *del_img;
 	GtkWidget *add_img;
+	GtkWidget *match_menu_in_addr;
+	GtkWidget *match_menu_not_in_addr;
 
 	cond_hbox = g_new0(CondHBox, 1);
 
@@ -518,24 +523,31 @@ CondHBox *prefs_filter_edit_cond_hbox_create(FilterCondEdit *cond_edit)
 	gtk_widget_show(match_type_optmenu);
 	gtk_box_pack_start(GTK_BOX(hbox), match_type_optmenu, FALSE, FALSE, 0);
 
+#define MATCH_MENUITEM_ADD(str, action)					    \
+{									    \
+	MENUITEM_ADD(menu, menuitem, str, action);			    \
+	g_signal_connect(G_OBJECT(menuitem), "activate",		    \
+			 G_CALLBACK(prefs_filter_match_activated_cb),	    \
+			 cond_hbox);					    \
+}
 	menu = gtk_menu_new();
 	gtk_widget_show(menu);
-	MENUITEM_ADD(menu, menuitem, _("contains"),
-		     PF_MATCH_CONTAIN);
-	MENUITEM_ADD(menu, menuitem, _("doesn't contain"),
-		     PF_MATCH_NOT_CONTAIN);
-	MENUITEM_ADD(menu, menuitem, _("is"),
-		     PF_MATCH_EQUAL);
-	MENUITEM_ADD(menu, menuitem, _("is not"),
-		     PF_MATCH_NOT_EQUAL);
+	MATCH_MENUITEM_ADD(_("contains"),               PF_MATCH_CONTAIN);
+	MATCH_MENUITEM_ADD(_("doesn't contain"),        PF_MATCH_NOT_CONTAIN);
+	MATCH_MENUITEM_ADD(_("is"),                     PF_MATCH_EQUAL);
+	MATCH_MENUITEM_ADD(_("is not"),                 PF_MATCH_NOT_EQUAL);
 #if defined(USE_ONIGURUMA) || defined(HAVE_REGCOMP)
-	MENUITEM_ADD(menu, menuitem, _("match to regex"),
-		     PF_MATCH_REGEX);
-	MENUITEM_ADD(menu, menuitem, _("doesn't match to regex"),
-		     PF_MATCH_NOT_REGEX);
+	MATCH_MENUITEM_ADD(_("match to regex"),         PF_MATCH_REGEX);
+	MATCH_MENUITEM_ADD(_("doesn't match to regex"), PF_MATCH_NOT_REGEX);
 #endif
+	MATCH_MENUITEM_ADD(_("is in addressbook"),      PF_MATCH_IN_ADDRESSBOOK);
+	match_menu_in_addr = menuitem;
+	MATCH_MENUITEM_ADD(_("is not in addressbook"),  PF_MATCH_NOT_IN_ADDRESSBOOK);
+	match_menu_not_in_addr = menuitem;
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(match_type_optmenu), menu);
 
+#undef MATCH_MENUITEM_ADD
+
 	size_match_optmenu = gtk_option_menu_new();
 	gtk_widget_show(size_match_optmenu);
 	gtk_box_pack_start(GTK_BOX(hbox), size_match_optmenu, FALSE, FALSE, 0);
@@ -572,7 +584,7 @@ CondHBox *prefs_filter_edit_cond_hbox_create(FilterCondEdit *cond_edit)
 	gtk_widget_show(key_entry);
 	gtk_box_pack_start(GTK_BOX(hbox), key_entry, TRUE, TRUE, 0);
 
-	spin_btn_adj = gtk_adjustment_new(0, 0, 99999, 1, 10, 100);
+	spin_btn_adj = gtk_adjustment_new(0, 0, 99999, 1, 10, 0);
 	spin_btn = gtk_spin_button_new(GTK_ADJUSTMENT(spin_btn_adj), 1, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), spin_btn, FALSE, FALSE, 0);
 	gtk_widget_set_size_request(spin_btn, 64, -1);
@@ -612,6 +624,8 @@ CondHBox *prefs_filter_edit_cond_hbox_create(FilterCondEdit *cond_edit)
 	cond_hbox->key_entry = key_entry;
 	cond_hbox->spin_btn = spin_btn;
 	cond_hbox->label = label;
+	cond_hbox->match_menu_in_addr = match_menu_in_addr;
+	cond_hbox->match_menu_not_in_addr = match_menu_not_in_addr;
 	cond_hbox->del_btn = del_btn;
 	cond_hbox->add_btn = add_btn;
 	cond_hbox->cur_type = PF_COND_HEADER;
@@ -702,7 +716,7 @@ ActionHBox *prefs_filter_edit_action_hbox_create(void)
 	gtk_box_pack_start(GTK_BOX(hbox), folder_entry, TRUE, TRUE, 0);
 
 	folder_img = stock_pixbuf_widget(rule_edit_window.window,
-					 STOCK_PIXMAP_DIR_OPEN);
+					 STOCK_PIXMAP_FOLDER_OPEN);
 	gtk_widget_show(folder_img);
 
 	folder_sel_btn = gtk_button_new();
@@ -841,6 +855,12 @@ void prefs_filter_edit_cond_hbox_set(CondHBox *hbox, FilterCond *cond)
 			else
 				match_type = PF_MATCH_REGEX;
 			break;
+		case FLT_IN_ADDRESSBOOK:
+			if (FLT_IS_NOT_MATCH(cond->match_flag))
+				match_type = PF_MATCH_NOT_IN_ADDRESSBOOK;
+			else
+				match_type = PF_MATCH_IN_ADDRESSBOOK;
+			break;
 		}
 		break;
 	case FLT_COND_UNREAD:
@@ -895,6 +915,10 @@ void prefs_filter_edit_cond_hbox_set(CondHBox *hbox, FilterCond *cond)
 		gtk_option_menu_set_history
 			(GTK_OPTION_MENU(hbox->status_match_optmenu),
 			 status_type);
+
+	if (match_type == PF_MATCH_IN_ADDRESSBOOK ||
+	    match_type == PF_MATCH_NOT_IN_ADDRESSBOOK)
+		gtk_widget_hide(hbox->key_entry);
 }
 
 void prefs_filter_edit_action_hbox_set(ActionHBox *hbox, FilterAction *action)
@@ -977,6 +1001,8 @@ void prefs_filter_edit_cond_hbox_select(CondHBox *hbox, CondMenuType type,
 
 void prefs_filter_edit_set_cond_hbox_widgets(CondHBox *hbox, CondMenuType type)
 {
+	MatchMenuType match_type;
+
 	switch (type) {
 	case PF_COND_HEADER:
 	case PF_COND_TO_OR_CC:
@@ -986,9 +1012,27 @@ void prefs_filter_edit_set_cond_hbox_widgets(CondHBox *hbox, CondMenuType type)
 		gtk_widget_hide(hbox->size_match_optmenu);
 		gtk_widget_hide(hbox->age_match_optmenu);
 		gtk_widget_hide(hbox->status_match_optmenu);
-		gtk_widget_show(hbox->key_entry);
+		match_type = menu_get_option_menu_active_index
+			(GTK_OPTION_MENU(hbox->match_type_optmenu));
+		if (match_type == PF_MATCH_IN_ADDRESSBOOK ||
+		    match_type == PF_MATCH_NOT_IN_ADDRESSBOOK)
+			gtk_widget_hide(hbox->key_entry);
+		else
+			gtk_widget_show(hbox->key_entry);
 		gtk_widget_hide(hbox->spin_btn);
 		gtk_widget_hide(hbox->label);
+		if (type == PF_COND_HEADER || type == PF_COND_TO_OR_CC) {
+			gtk_widget_show(hbox->match_menu_in_addr);
+			gtk_widget_show(hbox->match_menu_not_in_addr);
+		} else {
+			gtk_widget_hide(hbox->match_menu_in_addr);
+			gtk_widget_hide(hbox->match_menu_not_in_addr);
+			if (match_type == PF_MATCH_IN_ADDRESSBOOK ||
+			    match_type == PF_MATCH_NOT_IN_ADDRESSBOOK) {
+				gtk_option_menu_set_history(GTK_OPTION_MENU(hbox->match_type_optmenu), 0);
+				gtk_widget_show(hbox->key_entry);
+			}
+		}
 		break;
 	case PF_COND_CMD_TEST:
 		gtk_widget_hide(hbox->match_type_optmenu);
@@ -1703,6 +1747,13 @@ FilterCond *prefs_filter_edit_cond_hbox_to_cond(CondHBox *hbox,
 		match_type = FLT_REGEX;
 		match_flag |= FLT_NOT_MATCH;
 		break;
+	case PF_MATCH_IN_ADDRESSBOOK:
+		match_type = FLT_IN_ADDRESSBOOK;
+		break;
+	case PF_MATCH_NOT_IN_ADDRESSBOOK:
+		match_type = FLT_IN_ADDRESSBOOK;
+		match_flag |= FLT_NOT_MATCH;
+		break;
 	default:
 		break;
 	}
@@ -1905,7 +1956,7 @@ static FilterRule *prefs_filter_edit_dialog_to_rule(void)
 	FilterRule *rule = NULL;
 	GSList *cur;
 	const gchar *rule_name;
-	FilterBoolOp bool_op = FLT_OR;
+	FilterBoolOp bool_op;
 	GSList *cond_list = NULL;
 	GSList *action_list = NULL;
 	GtkWidget *bool_op_menuitem;
@@ -2035,6 +2086,21 @@ static void prefs_filter_cond_activated_cb(GtkWidget *widget, gpointer data)
 	}
 }
 
+static void prefs_filter_match_activated_cb(GtkWidget *widget, gpointer data)
+{
+	CondHBox *hbox = (CondHBox *)data;
+	GtkWidget *cond_type_menuitem;
+	CondMenuType cond_menu_type;
+
+	cond_type_menuitem = gtk_menu_get_active
+		(GTK_MENU(gtk_option_menu_get_menu
+			(GTK_OPTION_MENU(hbox->cond_type_optmenu))));
+	cond_menu_type = GPOINTER_TO_INT
+		(g_object_get_data(G_OBJECT(cond_type_menuitem), MENU_VAL_ID));
+
+	prefs_filter_edit_set_cond_hbox_widgets(hbox, cond_menu_type);
+}
+
 static void prefs_filter_action_activated_cb(GtkWidget *widget, gpointer data)
 {
 	ActionHBox *hbox = (ActionHBox *)data;
