@@ -298,6 +298,13 @@ gint rpop3_account(PrefsAccount *account)
 	while (!rpop3_window.finished)
 		gtk_main_iteration();
 
+	if (POP3_SESSION(session)->error_val == PS_AUTHFAIL &&
+	    account->tmp_pass) {
+		debug_print("rpop3_account: remove temporary password because of authentication failure\n");
+		g_free(account->tmp_pass);
+		account->tmp_pass = NULL;
+	}
+
 	rpop3_idle(FALSE);
 	session_destroy(session);
 	rpop3_clear_list();
@@ -608,6 +615,8 @@ static void rpop3_clear_list(void)
 
 static gboolean rpop3_ping_cb(gpointer data)
 {
+	gdk_threads_enter();
+
 	if (rpop3_window.ping_tag > 0) {
 		g_source_remove(rpop3_window.ping_tag);
 		rpop3_window.ping_tag = 0;
@@ -615,6 +624,8 @@ static gboolean rpop3_ping_cb(gpointer data)
 	if (rpop3_window.session->state == POP3_IDLE)
 		rpop3_noop_send(rpop3_window.session);
 
+	gdk_threads_leave();
+
 	return FALSE;
 }
 
@@ -629,8 +640,9 @@ static void rpop3_idle(gboolean is_idle)
 		rpop3_window.session->state = POP3_IDLE;
 		if (POP3_PING_ITV < prefs_common.io_timeout_secs)
 			rpop3_window.ping_tag =
-				g_timeout_add(POP3_PING_ITV * 1000,
-					      rpop3_ping_cb, NULL);
+				g_timeout_add_full(G_PRIORITY_LOW,
+						   POP3_PING_ITV * 1000,
+						   rpop3_ping_cb, NULL, NULL);
 	}
 }
 
@@ -718,7 +730,7 @@ static gint rpop3_retr_recv(Pop3Session *session, FILE *fp, guint len)
 {
 	gchar *file;
 	MsgInfo *msginfo;
-	MsgFlags flags = {0, 0};
+	MsgFlags flags = {MSG_NEW|MSG_UNREAD, MSG_RECEIVED};
 	MessageView *msgview;
 
 	file = get_tmp_file();
@@ -743,7 +755,7 @@ static gint rpop3_retr_recv(Pop3Session *session, FILE *fp, guint len)
 			return -1;
 		}
 
-		if (folder_item_add_msg(inbox, file, NULL, FALSE) < 0) {
+		if (folder_item_add_msg(inbox, file, &flags, FALSE) < 0) {
 			session->error_val = PS_IOERR;
 			return -1;
 		}
@@ -794,6 +806,8 @@ static gint rpop3_session_recv_msg(Session *session, const gchar *msg)
 	gint val = PS_SUCCESS;
 	const gchar *body;
 
+	gdk_threads_enter();
+
 	body = msg;
 	if (pop3_session->state != POP3_GETRANGE_UIDL_RECV &&
 	    pop3_session->state != POP3_GETSIZE_LIST_RECV) {
@@ -801,6 +815,7 @@ static gint rpop3_session_recv_msg(Session *session, const gchar *msg)
 		if (val != PS_SUCCESS) {
 			if (val != PS_NOTSUPPORTED) {
 				pop3_session->state = POP3_ERROR;
+				gdk_threads_leave();
 				return -1;
 			}
 		}
@@ -829,14 +844,14 @@ static gint rpop3_session_recv_msg(Session *session, const gchar *msg)
 			val = pop3_getauth_user_send(pop3_session);
 		break;
 #if USE_SSL
-        case POP3_STLS:
-                if ((val = pop3_stls_recv(pop3_session)) != PS_SUCCESS)
-                        return -1;
-                if (pop3_session->ac_prefs->use_apop_auth)
-                        val = pop3_getauth_apop_send(pop3_session);
-                else
-                        val = pop3_getauth_user_send(pop3_session);
-                break;
+	case POP3_STLS:
+		if ((val = pop3_stls_recv(pop3_session)) != PS_SUCCESS)
+			break;
+		if (pop3_session->ac_prefs->use_apop_auth)
+			val = pop3_getauth_apop_send(pop3_session);
+		else
+			val = pop3_getauth_user_send(pop3_session);
+		break;
 #endif
 	case POP3_GETAUTH_USER:
 		val = pop3_getauth_pass_send(pop3_session);
@@ -846,52 +861,52 @@ static gint rpop3_session_recv_msg(Session *session, const gchar *msg)
 		rpop3_status_label_set(_("Getting the number of messages..."));
 		val = pop3_getrange_stat_send(pop3_session);
 		break;
-        case POP3_GETRANGE_STAT:
-                if ((val = pop3_getrange_stat_recv(pop3_session, body)) < 0)
-                        return -1;
-                if (pop3_session->count > 0)
-                        val = pop3_getrange_uidl_send(pop3_session);
-                else {
+	case POP3_GETRANGE_STAT:
+		if ((val = pop3_getrange_stat_recv(pop3_session, body)) < 0)
+			break;
+		if (pop3_session->count > 0)
+			val = pop3_getrange_uidl_send(pop3_session);
+		else {
 			rpop3_status_label_set(_("No message"));
-                        val = pop3_logout_send(pop3_session);
+			val = pop3_logout_send(pop3_session);
 		}
-                break;
-        case POP3_GETRANGE_LAST:
-                if (val == PS_NOTSUPPORTED)
-                        pop3_session->error_val = PS_SUCCESS;
-                else if ((val = pop3_getrange_last_recv
-                                (pop3_session, body)) < 0)
-                        return -1;
-                if (pop3_session->cur_msg > 0)
-                        val = pop3_getsize_list_send(pop3_session);
-                else {
+		break;
+	case POP3_GETRANGE_LAST:
+		if (val == PS_NOTSUPPORTED)
+			pop3_session->error_val = PS_SUCCESS;
+		else if ((val = pop3_getrange_last_recv
+			(pop3_session, body)) < 0)
+			break;
+		if (pop3_session->cur_msg > 0)
+			val = pop3_getsize_list_send(pop3_session);
+		else {
 			rpop3_status_label_set(_("No message"));
-                        val = pop3_logout_send(pop3_session);
+			val = pop3_logout_send(pop3_session);
 		}
-                break;
-        case POP3_GETRANGE_UIDL:
-                if (val == PS_NOTSUPPORTED) {
-                        pop3_session->error_val = PS_SUCCESS;
-                        val = pop3_getrange_last_send(pop3_session);
-                } else {
-                        pop3_session->state = POP3_GETRANGE_UIDL_RECV;
-                        val = session_recv_data(session, 0, ".\r\n");
-                }
-                break;
-        case POP3_GETSIZE_LIST:
-                pop3_session->state = POP3_GETSIZE_LIST_RECV;
-                val = session_recv_data(session, 0, ".\r\n");
-                break;
+		break;
+	case POP3_GETRANGE_UIDL:
+		if (val == PS_NOTSUPPORTED) {
+			pop3_session->error_val = PS_SUCCESS;
+			val = pop3_getrange_last_send(pop3_session);
+		} else {
+			pop3_session->state = POP3_GETRANGE_UIDL_RECV;
+			val = session_recv_data(session, 0, ".\r\n");
+		}
+		break;
+	case POP3_GETSIZE_LIST:
+		pop3_session->state = POP3_GETSIZE_LIST_RECV;
+		val = session_recv_data(session, 0, ".\r\n");
+		break;
 	case POP3_TOP:
 		pop3_session->state = POP3_TOP_RECV;
-                val = session_recv_data_as_file(session, 0, ".\r\n");
+		val = session_recv_data_as_file(session, 0, ".\r\n");
 		break;
-        case POP3_RETR:
-                pop3_session->state = POP3_RETR_RECV;
-                val = session_recv_data_as_file(session, 0, ".\r\n");
-                break;
-        case POP3_DELETE:
-                val = rpop3_delete_recv(pop3_session);
+	case POP3_RETR:
+		pop3_session->state = POP3_RETR_RECV;
+		val = session_recv_data_as_file(session, 0, ".\r\n");
+		break;
+	case POP3_DELETE:
+		val = rpop3_delete_recv(pop3_session);
 		if (val != PS_SUCCESS)
 			break;
 		if (rpop3_window.delete_cur + 1 < rpop3_window.delete_array->len) {
@@ -904,21 +919,23 @@ static gint rpop3_session_recv_msg(Session *session, const gchar *msg)
 			rpop3_window.delete_array = NULL;
 			rpop3_window.delete_cur = 0;
 			rpop3_idle(TRUE);
-		break;
 		}
-                break;
+		break;
 	case POP3_NOOP:
 		rpop3_idle(TRUE);
 		break;
-        case POP3_LOGOUT:
-                pop3_session->state = POP3_DONE;
-                session_disconnect(session);
-                break;
+	case POP3_LOGOUT:
+		pop3_session->state = POP3_DONE;
+		session_disconnect(session);
+		break;
 	case POP3_ERROR:
 	default:
-		return -1;
+		val = -1;
+		break;
 	}
 
+	gdk_threads_leave();
+
 	if (val == PS_SUCCESS)
 		return 0;
 	else
@@ -930,6 +947,9 @@ static gint rpop3_session_recv_data_finished(Session *session, guchar *data,
 {
         Pop3Session *pop3_session = POP3_SESSION(session);
         Pop3ErrorValue val = PS_SUCCESS;
+	gint ret = 0;
+
+	gdk_threads_enter();
 
         switch (pop3_session->state) {
         case POP3_GETRANGE_UIDL_RECV:
@@ -940,9 +960,9 @@ static gint rpop3_session_recv_data_finished(Session *session, guchar *data,
 				pop3_logout_send(rpop3_window.session);
 			} else
 				pop3_getsize_list_send(pop3_session);
-                } else
-                        return -1;
-                break;
+		} else
+			ret = -1;
+		break;
         case POP3_GETSIZE_LIST_RECV:
                 val = pop3_getsize_list_recv(pop3_session, (gchar *)data, len);
                 if (val == PS_SUCCESS) {
@@ -958,30 +978,38 @@ static gint rpop3_session_recv_data_finished(Session *session, guchar *data,
 				rpop3_top_send(pop3_session);
 			}
 		} else
-                        return -1;
-                break;
-        case POP3_ERROR:
-        default:
-                return -1;
-        }
+			ret = -1;
+		break;
+	case POP3_ERROR:
+	default:
+		ret = -1;
+		break;
+	}
 
-        return 0;
+	gdk_threads_leave();
+
+	return ret;
 }
 
 static gint rpop3_session_recv_data_as_file_finished(Session *session,
 						     FILE *fp, guint len)
 {
         Pop3Session *pop3_session = POP3_SESSION(session);
+	gint ret = 0;
+
+	gdk_threads_enter();
 
         switch (pop3_session->state) {
 	case POP3_RETR_RECV:
-        	if (rpop3_retr_recv(pop3_session, fp, len) < 0)
-                	return -1;
+        	if (rpop3_retr_recv(pop3_session, fp, len) < 0) {
+                	ret = -1;
+			break;
+		}
 		if (rpop3_window.recv_array) {
 			if (rpop3_window.recv_cur + 1 < rpop3_window.recv_array->len) {
 				rpop3_window.recv_cur++;
 				if (rpop3_retr_send(pop3_session) < 0)
-					return -1;
+					ret = -1;
 			} else {
 				rpop3_status_label_set(_("Retrieved %d messages"), rpop3_window.recv_cur + 1);
 				g_array_free(rpop3_window.recv_array, TRUE);
@@ -1027,13 +1055,16 @@ static gint rpop3_session_recv_data_as_file_finished(Session *session,
 				rpop3_idle(TRUE);
 			}
 		} else
-			return -1;
+			ret = -1;
 		break;
 	default:
-		return -1;
+		ret = -1;
+		break;
 	}
 
-        return 0;
+	gdk_threads_leave();
+
+	return ret;
 }
 
 static gint window_deleted(GtkWidget *widget, GdkEventAny *event,
@@ -1080,7 +1111,7 @@ static void rpop3_recv(GtkButton *button, gpointer data)
 		gtk_tree_model_get(model, &iter, COL_NUMBER, &num,
 				   COL_DELETED, &deleted, -1);
 		if (!deleted) {
-			debug_print("rpop3_recv: recieving message %d\n", num);
+			debug_print("rpop3_recv: receiving message %d\n", num);
 			g_array_append_val(array, num);
 		}
 	}
