@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,6 +56,7 @@
 #include "displayheader.h"
 #include "filesel.h"
 #include "alertpanel.h"
+#include "plugin.h"
 
 typedef struct _RemoteURI	RemoteURI;
 
@@ -196,6 +197,9 @@ static void textview_populate_popup		(GtkWidget	*widget,
 static void textview_popup_menu_activate_open_uri_cb
 						(GtkMenuItem	*menuitem,
 						 gpointer	 data);
+static void textview_popup_menu_activate_reply_cb
+						(GtkMenuItem	*menuitem,
+						 gpointer	 data);
 static void textview_popup_menu_activate_add_address_cb
 						(GtkMenuItem	*menuitem,
 						 gpointer	 data);
@@ -1970,6 +1974,14 @@ static gboolean textview_visibility_notify(GtkWidget *widget,
 	gtk_widget_show(menuitem);					\
 }
 
+#define ADD_MENU_SEPARATOR()						\
+{									\
+	menuitem = gtk_menu_item_new();					\
+	gtk_widget_set_sensitive(menuitem, FALSE);			\
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);		\
+	gtk_widget_show(menuitem);					\
+}
+
 static void textview_populate_popup(GtkWidget *widget, GtkMenu *menu,
 				    TextView *textview)
 {
@@ -1980,6 +1992,7 @@ static void textview_populate_popup(GtkWidget *widget, GtkMenu *menu,
 	gboolean on_link;
 	RemoteURI *uri;
 	GdkPixbuf *pixbuf;
+	gchar *selected_text;
 
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
 
@@ -2000,13 +2013,17 @@ static void textview_populate_popup(GtkWidget *widget, GtkMenu *menu,
 		ADD_MENU(_("Sa_ve this image as..."),
 			 textview_popup_menu_activate_image_cb);
 	}
+
+	selected_text = gtkut_text_view_get_selection(GTK_TEXT_VIEW(widget));
+
+	uri = NULL;
 	on_link = textview_get_link_tag_bounds(textview, &iter, &start, &end);
 	if (!on_link)
-		return;
+		goto finish;
 
 	uri = textview_get_uri(textview, &start, &end);
 	if (!uri)
-		return;
+		goto finish;
 
 	separator = gtk_separator_menu_item_new();
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), separator);
@@ -2015,6 +2032,9 @@ static void textview_populate_popup(GtkWidget *widget, GtkMenu *menu,
 	if (!g_ascii_strncasecmp(uri->uri, "mailto:", 7)) {
 		ADD_MENU(_("Compose _new message"),
 			 textview_popup_menu_activate_open_uri_cb);
+		ADD_MENU(_("R_eply to this address"),
+			 textview_popup_menu_activate_reply_cb);
+		ADD_MENU_SEPARATOR();
 		ADD_MENU(_("Add to address _book..."),
 			 textview_popup_menu_activate_add_address_cb);
 		ADD_MENU(_("Copy this add_ress"),
@@ -2025,6 +2045,14 @@ static void textview_populate_popup(GtkWidget *widget, GtkMenu *menu,
 		ADD_MENU(_("Copy this _link"),
 			 textview_popup_menu_activate_copy_cb);
 	}
+
+finish:
+	syl_plugin_signal_emit("textview-menu-popup", menu,
+			       GTK_TEXT_VIEW(widget),
+			       uri ? uri->uri : NULL,
+			       selected_text,
+			       textview->messageview->msginfo);
+	g_free(selected_text);
 }
 
 static void textview_popup_menu_activate_open_uri_cb(GtkMenuItem *menuitem,
@@ -2054,6 +2082,53 @@ static void textview_popup_menu_activate_open_uri_cb(GtkMenuItem *menuitem,
 		open_uri(uri->uri, prefs_common.uri_cmd);
 }
 
+static void textview_popup_menu_activate_reply_cb(GtkMenuItem *menuitem,
+						  gpointer data)
+{
+	RemoteURI *uri = (RemoteURI *)data;
+	TextView *textview;
+
+	g_return_if_fail(uri != NULL);
+
+	if (!uri->uri)
+		return;
+
+	textview = g_object_get_data(G_OBJECT(menuitem), "textview");
+	g_return_if_fail(textview != NULL);
+
+	if (!g_ascii_strncasecmp(uri->uri, "mailto:", 7)) {
+		MsgInfo *msginfo = textview->messageview->msginfo;
+		ComposeMode mode = COMPOSE_REPLY;
+		gchar *text;
+		GList *list;
+		Compose *compose;
+
+		g_return_if_fail(msginfo != NULL);
+
+		if (prefs_common.reply_with_quote)
+			mode |= COMPOSE_WITH_QUOTE;
+
+		text = gtkut_text_view_get_selection
+			(GTK_TEXT_VIEW(textview->text));
+		if (text && *text == '\0') {
+			g_free(text);
+			text = NULL;
+		}
+
+		compose_reply(msginfo, msginfo->folder, mode, text);
+		list = compose_get_compose_list();
+		list = g_list_last(list);
+		if (list && list->data) {
+			compose = (Compose *)list->data;
+			compose_block_modified(compose);
+			compose_entry_set(compose, uri->uri + 7,
+					  COMPOSE_ENTRY_TO);
+			compose_unblock_modified(compose);
+		}
+		g_free(text);
+	}
+}
+
 static void textview_popup_menu_activate_add_address_cb(GtkMenuItem *menuitem,
 							gpointer data)
 {
